{ 
  type token =
    | Id_token of (string)
    | SELECT_token
    | FROM_token
    | WHERE_token
    | TABLE_token
    | CREATE_token
    | DATABASE_token
    | DROP_token
    | INSERT_token
    | DELETE_token
    | UPDATE_token
    | VALUES_token
    | SUM_token
    | AVG_token
    | EQUAL_token
    | NOTEQUAL_token
    | LParen_token
    | RParen_token
    | Integer_token of int
    | StringLiteral_token of (string)
    | EOF_token
}

let numeric = ['0' - '9']
let letter =['a' - 'z' 'A' - 'Z']

rule token = parse
  | "SELECT"              {SELECT_token }
  | "FROM"                {FROM_token }
  | "WHERE"               {WHERE_token }
  | "TABLE"               {TABLE_token }
  | "CREATE"              {CREATE_token }
  | "DATABASE"            {DATABASE_token }
  | "DROP"                {DROP_token }
  | "INSERT"              {INSERT_token }
  | "DELETE"              {DELETE_token }
  | "UPDATE"              {UPDATE_token }
  | "VALUES"              {VALUES_token }
  | "SUM"                 {SUM_token }
  | "AVG"                 {AVG_token }
  | "<>"                  {NOTEQUAL_token }
  | "="                   {EQUAL_token }
  | "("                   {LParen_token }
  | ")"                   {RParen_token }
  | letter (letter|numeric|"_")* as id  
                          {Id_token id }
  | (numeric)* as i       {Integer_token (int_of_string i)  }
  | ''' (([^'"''\\'] | '\\'_)* as s) '''  { StringLiteral_token (s)  }
  | eof                   {EOF_token }
  | [' ' '\t']            {token lexbuf}

{ let lextest s = token (Lexing.from_string s)}
